@@ -11,6 +11,8 @@ from asyncio import (
     Task,
     ensure_future,
     get_event_loop,
+    new_event_loop,
+    set_event_loop,
     sleep,
 )
 from subprocess import Popen
@@ -431,6 +433,12 @@ class Application(Generic[_AppResult]):
         """
         Thread safe way of sending a repaint trigger to the input event loop.
         """
+        if not self._is_running:
+            # Don't schedule a redraw if we're not running.
+            # Otherwise, `get_event_loop()` in `call_soon_threadsafe` can fail.
+            # See: https://github.com/dbcli/mycli/issues/797
+            return
+
         # Never schedule a second redraw, when a previous one has not yet been
         # executed. (This should protect against other threads calling
         # 'invalidate' many times, resulting in 100% CPU.)
@@ -616,7 +624,7 @@ class Application(Generic[_AppResult]):
             # pressed, we start a 'flush' timer for flushing our escape key. But
             # when any subsequent input is received, a new timer is started and
             # the current timer will be ignored.
-            flush_counter = 0
+            flush_task: Optional[asyncio.Task] = None
 
             # Reset.
             self.reset()
@@ -627,7 +635,7 @@ class Application(Generic[_AppResult]):
             self.key_processor.process_keys()
 
             def read_from_input() -> None:
-                nonlocal flush_counter
+                nonlocal flush_task
 
                 # Ignore when we aren't running anymore. This callback will
                 # removed from the loop next time. (It could be that it was
@@ -647,20 +655,17 @@ class Application(Generic[_AppResult]):
                 if self.input.closed:
                     f.set_exception(EOFError)
                 else:
-                    # Increase this flush counter.
-                    flush_counter += 1
-                    counter = flush_counter
-
                     # Automatically flush keys.
-                    self.create_background_task(auto_flush_input(counter))
+                    if flush_task:
+                        flush_task.cancel()
+                    flush_task = self.create_background_task(auto_flush_input())
 
-            async def auto_flush_input(counter: int) -> None:
+            async def auto_flush_input() -> None:
                 # Flush input after timeout.
                 # (Used for flushing the enter key.)
+                # This sleep can be cancelled, in that case we won't flush yet.
                 await sleep(self.ttimeoutlen)
-
-                if flush_counter == counter:
-                    flush_input()
+                flush_input()
 
             def flush_input() -> None:
                 if not self.is_done:
@@ -779,13 +784,30 @@ class Application(Generic[_AppResult]):
         """
         A blocking 'run' call that waits until the UI is finished.
 
+        This will start the current asyncio event loop. If no loop is set for
+        the current thread, then it will create a new loop.
+
         :param pre_run: Optional callable, which is called right after the
             "reset" of the application.
         :param set_exception_handler: When set, in case of an exception, go out
             of the alternate screen and hide the application, display the
             exception, and wait for the user to press ENTER.
         """
-        return get_event_loop().run_until_complete(self.run_async(pre_run=pre_run))
+        # We don't create a new event loop by default, because we want to be
+        # sure that when this is called multiple times, each call of `run()`
+        # goes through the same event loop. This way, users can schedule
+        # background-tasks that keep running across multiple prompts.
+        try:
+            loop = get_event_loop()
+        except RuntimeError:
+            # Possibly we are not running in the main thread, where no event
+            # loop is set by default. Or somebody called `asyncio.run()`
+            # before, which closes the existing event loop. We can create a new
+            # loop.
+            loop = new_event_loop()
+            set_event_loop(loop)
+
+        return loop.run_until_complete(self.run_async(pre_run=pre_run))
 
     def _handle_exception(self, loop, context: Dict[str, Any]) -> None:
         """
